"""Plugin utils

Utilities for handling Flaschengeist plugins
"""

import pkg_resources
from flaschengeist import logger
from flaschengeist.plugins import Plugin


def get_plugins() -> list[type[Plugin]]:
    """Get all installed plugins for Flaschengeist

    Returns:
        list of classes implementing `flaschengeist.plugins.Plugin`
    """
    logger.debug("Search for plugins")

    plugins = []
    for entry_point in pkg_resources.iter_entry_points("flaschengeist.plugins"):
        try:
            logger.debug(f"Found plugin: >{entry_point.name}<")
            plugin_class = entry_point.load()
            if issubclass(plugin_class, Plugin):
                plugins.append(plugin_class)
        except TypeError:
            logger.error(f"Invalid entry point for plugin {entry_point.name} found.")
        except pkg_resources.DistributionNotFound:
            logger.warn(f"Requirements not fulfilled for {entry_point.name}")
            logger.debug("DistributionNotFound", exc_info=True)
    return plugins


def plugin_version(plugin: type[Plugin]) -> str:
    """Get version of plugin

    Returns the version of a plugin, if plugin does not set the
    version property, the version of the package providing the
    plugin is taken.

    Args:
        plugin: Plugin or Plugin class

    Returns:
        Version as string
    """
    if plugin.version:
        return plugin.version
    return pkg_resources.get_distribution(plugin.__module__.split(".", 1)[0]).version
